// -*- C++ -*-
// $RCSfile: threadstate.C,v $
// $Revision: 1.29 $
// $Author: redwing $
// $Date: 2004/10/29 14:01:09 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */


#include <oofconfig.h>
#include <Python.h>

#include "common/IO/progressbar.h" //For the KeyHolder...
#include "common/lock.h"
#include "common/ooferror.h"
#include "threadstate.h"
#include <iostream>
#include <pthread.h>
#include <string>
#include <vector>

Lock lockListOfStates;

std::vector<ThreadState*> &listOfStates() {
  static std::vector<ThreadState*> list;
  //std::cerr << "std::vector<ThreadState*> &listOfStates()" << std::endl;
  return list;
}

ThreadState *findThreadState() {
  ThreadID currentThread;
  std::vector<ThreadState*> &list = listOfStates();

  KeyHolder hldr(lockListOfStates); 

  for(int i=0; i<list.size(); i++)
    {
      // std::cerr << "ThreadState *findThreadState()" << std::endl;
      if(list[i]->get_thread_ID() == currentThread)
	return list[i];
    }
  //  std::cerr << "returning new ThreadState object" << std::endl;
  return new ThreadState();
}

int findThreadNumber() {
  ThreadState *ts = findThreadState();
  if(ts)
    return ts->id();
  return 0;
}

ThreadID::ThreadID() {
  _ID = pthread_self();
}

int operator==(ThreadID &t1, ThreadID &t2) {
  return pthread_equal(t1.get_ID(), t2.get_ID());
}

static int uniqueThreadSafeID() {
  static int counter = 0;
  static pthread_mutex_t lock = PTHREAD_MUTEX_INITIALIZER;
  pthread_mutex_lock(&lock);
  int id = counter++;
  pthread_mutex_unlock(&lock);
  return id;
}


ThreadState::ThreadState()
  : _state(0),
    _id(uniqueThreadSafeID()),
    counter(1), 
    defunct(0)
{
  /*
    Do not lock the ThreadState constructor as long as findThreadState is the 
    only place where ThreadState objects are constructed. If ThreadState is
    constructed in another place, then the locking architecture should be 
    rethought.
  */
//   std::cerr << "creating ThreadState " << _id << std::endl;
  listOfStates().push_back(this);
  int old_cancel_state;
  pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, &old_cancel_state);
  int old_cancel_type;
  pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, &old_cancel_type);
}

ThreadState::~ThreadState() {
  remove_from_list();
}


void ThreadState::releasePyLock() {
  counter--;
  if(counter == 0) {
    // PyEval_SaveThread returns the state of the thread and
    // releases the Python interpreter lock.
    _state = PyEval_SaveThread();

  }
}

void ThreadState::acquirePyLock() {
  counter++;
  if(counter == 1) {
    // PyEval_RestoreThread acquires the Python interpreter lock and
    // restores the saved state of the thread.
    PyEval_RestoreThread(_state);
  }
}

ThreadID & ThreadState::get_thread_ID() {
  _ID.get_ID();
  return _ID;
}

void  ThreadState::remove_from_list()
{
  std::vector<ThreadState*> &list = listOfStates();
  KeyHolder hldr(lockListOfStates);
  for(std::vector<ThreadState*>::iterator i=list.begin(); i<list.end(); ++i) {
    if(*i == this)
      {
	//std::cerr <<  "removing ThreadState from list" << std::endl;
	list.erase(i);
	return;
      }
  }

}


void cleanList()
{
  std::vector<ThreadState*> &list = listOfStates();
  int j = 0;
  int size = list.size();
  KeyHolder hldr(lockListOfStates);
  for(std::vector<ThreadState*>::iterator i=list.begin(); i<list.end(); ++i) {
    if((*i)->get_defunct() != 1)
      {
	//list.erase(i);
	list[j] = *i;
	j++;
	//std::cerr << "keeping item" << std::endl;
      }
  }
  list.resize(j);
}

int operator==(ThreadState &t1, ThreadState &t2) {
  return t1.get_thread_ID() == t2.get_thread_ID();
}


void releasePyLock() {
  findThreadState()->releasePyLock();
}

void acquirePyLock() {
  findThreadState()->acquirePyLock();
}

//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//

// Create a ThreadState object for the main thread.  This is created
// when the oof module is loaded by Python.

static ThreadState mainthreadstate;


void cancelThread(ThreadState &tobecancelled) {
  if (tobecancelled == mainthreadstate)
    {
      //std::cerr << "NOT cancelling thread" << std::endl;
      return; //make sure that the main thread is NOT stopped by this function
    }
  else
    {
      //std::cerr << "cancelThread: Cancelling thread " << tobecancelled.id()
      //<< " " << tobecancelled.get_thread_ID().get_ID() << std::endl;
      ThreadID id = tobecancelled.get_thread_ID();
      pthread_cancel(id.get_ID());
    }
  
}

void testcancel() {
  pthread_testcancel();
}
