# -*- python -*-
# $RCSfile: adaptivemeshrefineWidget.py,v $
# $Revision: 1.4 $
# $Author: langer $
# $Date: 2005/05/13 18:44:10 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 


from oof2.SWIG.common import switchboard
from oof2.common import debug
from oof2.common import guitop
from oof2.common.IO.GUI import chooser
from oof2.common.IO.GUI import parameterwidgets
from oof2.common.IO.GUI import whowidget
from oof2.engine import errorestimator
from oof2.engine import revertmesh
from oof2.engine import skeletoncontext
from oof2.engine.IO.GUI import meshparamwidgets
import oof2.engine.mesh

from gtk import *
import string

skeletonContexts = skeletoncontext.skeletonContexts

class AMRWhoParameterWidget(whowidget.WhoParameterWidget):
    def __init__(self, whoclass, value=None, scope=None):
        self.skelwidget = scope.findWidget(
            lambda x: isinstance(x, whowidget.WhoWidget)
            and x.whoclass is skeletonContexts)
        whowidget.WhoParameterWidgetBase.__init__(self, whoclass,
                                                  value=value, scope=scope)

        # switchboard callbacks
        self.sbcallbacks = [
            switchboard.requestCallback(self.skelwidget, self.skelwidgetChanged),
            switchboard.requestCallback(('who changed', 'Skeleton'), self.skelModified),
            switchboard.requestCallback('Mesh reverted', self.meshReverted)]
        
    def widgetCB(self, interactive):
        # Gives an "OK" sign only when the current skelect object is
        # the base skeleton of the to-be-refined mesh.
        try:
            myskel = skeletonContexts[self.skelwidget.get_value()].getObject()
            baseskel = oof2.engine.mesh.meshes[self.get_value()].getSkeleton()
            self.widgetChanged(myskel is baseskel, interactive)
        except KeyError:
            self.widgetChanged(0, interactive)

    # Callback for "skelwidget" change in Skeleton Page.
    def skelwidgetChanged(self, *args, **kwargs):
        self.widgetCB(1)

    # Callback for skeleton modification (modify, undo, redo).
    def skelModified(self, *args, **kwargs):
        self.widgetCB(1)
        
    # Callback for reverted mesh.
    def meshReverted(self, *args, **kwargs):
        self.widgetCB(1)

    def cleanUp(self):
        map(switchboard.removeCallback, self.sbcallbacks)
        whowidget.WhoParameterWidgetBase.cleanUp(self)
                        
def _AMRWhoParameter_makeWidget(self, scope=None):
    return AMRWhoParameterWidget(self.whoclass, self.value, scope=scope)

errorestimator.AMRWhoParameter.makeWidget = _AMRWhoParameter_makeWidget

##################################

class MeshSkeletonParameterWidget(parameterwidgets.ParameterWidget):
    def __init__(self, scope=None):
        self.meshwidget = scope.findWidget(
            lambda x: isinstance(x, whowidget.WhoWidget)
            and x.whoclass is oof2.engine.mesh.meshes)
        self.update()
        self.widget = chooser.ChooserWidget(self.nameset,
                                            callback=self.selection)
        parameterwidgets.ParameterWidget.__init__(self, self.widget.gtk, scope)
        self.widgetChanged(len(self.nameset) > 1, interactive=1)
        self.set_value(self.nameset[0])  # initial value
        # switchboard callbacks
        self.sbcallbacks= [switchboard.requestCallback(('new who', 'Mesh'), self.newMeshCreated)]
    def update(self):
        try:
            self.mesh = oof2.engine.mesh.meshes[self.meshwidget.get_value()]
            # no. of stored skeletons in the mesh
            nskels = len(self.mesh.skeleton_buffer)
            # make a nameset.
            # ["Base Skeleton 0", "Base Skeleton 1" ...]
            self.nameset = ["Base Skeleton "+`i` for i in range(nskels)]
        except KeyError:
            self.nameset = [""]
    def updateWidget(self):
        self.widget.update(self.nameset)
    def newMeshCreated(self, *args):
        self.update()
        self.updateWidget()
    def selection(self, gtkobj, name):
        self.value = name
        index = int(string.split(name)[-1])
        validity = self.mesh.skeleton_buffer.current() is not \
                   self.mesh.skeleton_buffer[index]
        self.widgetChanged(validity=validity, interactive=1)
    def get_value(self):
        return self.value
    def set_value(self, value):
        self.value = value
        self.widget.set_state(value)
    def cleanUp(self):
        map(switchboard.removeCallback, self.sbcallbacks)
        parameterwidgets.ParameterWidget.cleanUp(self)
        
def _MeshSkeletonParameter_makeWidget(self, scope=None):
    return MeshSkeletonParameterWidget(scope=scope)

revertmesh.MeshSkeletonParameter.makeWidget = _MeshSkeletonParameter_makeWidget

#########################################

class ZZFluxParameterWidget(meshparamwidgets.FluxParameterWidget):
    def __init__(self, param, scope):
        meshparamwidgets.FluxParameterWidget.__init__(self, param, scope)

        mesh = self.getMesh()
        if mesh:
            if mesh.has_solution():
                self.widgetChanged(1, interactive=1)
            else:
                self.widgetChanged(0, interactive=1)

        self.sbcallbacks += [switchboard.requestCallback(('new who', 'Mesh'),
                                                         self.meshChangedCB),
                             switchboard.requestCallback("mesh changed",
                                                         self.meshChangedCB)]

    def meshChangedCB(self, *args):
        mesh = self.getMesh()
        if mesh:
            if mesh.has_solution():
                self.widgetChanged(1, interactive=1)
            else:
                self.widgetChanged(0, interactive=1)
        
def _ZZFluxParameter_makeWidget(param, scope):
    return ZZFluxParameterWidget(param, scope)

errorestimator.ZZFluxParameter.makeWidget = _ZZFluxParameter_makeWidget
