# -*- python -*-
# $RCSfile: skeletonBoundaryPage.py,v $
# $Revision: 1.44 $
# $Author: langer $
# $Date: 2005/05/13 18:44:12 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

import gtk
from oof2.common import debug
from oof2.common import guitop
from oof2.common import labeltree
from oof2.common.IO import whoville
from oof2.common.IO.GUI import chooser
from oof2.common.IO.GUI import whowidget
from oof2.common.IO.GUI import oofGUI
from oof2.common.IO.GUI import oof_mainiteration
from oof2.common.IO.GUI import parameterwidgets
from oof2.engine import boundarybuilder
from oof2.engine import skeletoncontext
from oof2.engine.IO import boundarymenu
from oof2.SWIG.common import switchboard
from oof2.common import utils

class SkeletonBoundaryPage(oofGUI.MainPage):
    def __init__(self):
        
        oofGUI.MainPage.__init__(self, name="Skeleton Boundaries",
                                 ordering = 150,
                                 tip = "Create and orient boundaries.")

        mainbox = gtk.GtkVBox(spacing=2)
        self.gtk.add(mainbox)

        hbox = gtk.GtkHBox()
        mainbox.add(hbox, expand=0, fill=0)
        centerbox = gtk.GtkHBox(spacing=3)
        hbox.add(centerbox, expand=1, fill=0)
        
        self.skelwidget = whowidget.WhoWidget(whoville.getClass('Skeleton'),
                                              scope=self)
        switchboard.requestCallback(self.skelwidget, self.widgetChanged)
        label = gtk.GtkLabel('Microstructure=')
        label.set_alignment(1.0, 0.5)
        centerbox.add(label, expand=0, fill=0)
        centerbox.add(self.skelwidget.gtk[0], expand=0, fill=0)
        label = gtk.GtkLabel('Skeleton=')
        label.set_alignment(1.0, 0.5)
        centerbox.add(label, expand=0, fill=0)
        centerbox.add(self.skelwidget.gtk[1], expand=0, fill=0)

        mainpane = gtk.GtkHPaned()
        mainbox.add(mainpane)

        boundarylistframe = gtk.GtkFrame("Boundaries")
        mainpane.pack1(boundarylistframe, resize=0, shrink=0)
        
        innerframe = gtk.GtkFrame()
        innerframe.set_shadow_type(gtk.SHADOW_IN)
        boundarylistframe.add(innerframe)

        boundarylistbox = gtk.GtkVBox()
        innerframe.add(boundarylistbox)

        # List of all the boundaries.
        self.boundarylist = chooser.ChooserListWidget(
            callback=self.boundarylistCB, autoselect=0)
        boundarylistbox.add(self.boundarylist.gtk, expand=1, fill=1)

        boundarybuttonbox = gtk.GtkHBox()
        boundarylistbox.add(boundarybuttonbox, expand=0, fill=0)

        # Buttons that actually do stuff.
        self.newbutton = gtk.GtkButton("New...")
        self.newbutton.connect("clicked", self.newBoundaryCB)
        guitop.top().tooltip(self.newbutton,
                             "Construct a new boundary in the skeleton and associated meshes.")
        boundarybuttonbox.add(self.newbutton)

        self.editbutton = gtk.GtkButton("Modify...")
        self.editbutton.connect("clicked", self.modifyBoundaryCB)
        guitop.top().tooltip(self.editbutton,
                             "Modify the attributes of the selected boundary.")
        boundarybuttonbox.add(self.editbutton)
        
        self.renamebutton = gtk.GtkButton("Rename...")
        self.renamebutton.connect("clicked", self.renameBoundaryCB)
        guitop.top().tooltip(self.renamebutton,
                             "Rename the selected boundary.")
        boundarybuttonbox.add(self.renamebutton)

        self.deletebutton = gtk.GtkButton("Delete")
        self.deletebutton.connect("clicked", self.deleteBoundaryCB)
        guitop.top().tooltip(self.deletebutton,
                             "Delete the selected boundary from the skeleton and associated meshes.")
        boundarybuttonbox.add(self.deletebutton)
        
        # TODO LATER: Copying could be added here -- the scenario is
        # that a user may want to make a copy of a boundary, and then
        # edit one of the copies.  Currently boundary editing is
        # primitive (one can only add/remove components), but when
        # visual pointy-clicky boundary editing is added, copying will
        # make sense.

        infoframe = gtk.GtkFrame("Boundary data")
        mainpane.pack2(infoframe, resize=1, shrink=1)

        infowindow = gtk.GtkScrolledWindow()
        infoframe.add(infowindow)
        infowindow.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        
        self.infotext = gtk.GtkText()
        self.infotext.set_word_wrap(1)
        infowindow.add_with_viewport(self.infotext)
        
        # Catches push events *after* the boundaries have been
        # propagated, and also undo/redo events.  "who changed" is
        # too early.
        switchboard.requestCallback("new boundary configuration",
                                    self.newBdyConfigCB)
        switchboard.requestCallback("new boundary created", self.newBdyCB)
        switchboard.requestCallback("boundary removed", self.newBdyCB)
        switchboard.requestCallback("boundary renamed", self.newBdyCB)
        self.selectsignals = [
            switchboard.requestCallback("boundary selected",
                                        self.bdySelectedCB),
            switchboard.requestCallback("boundary unselected",
                                        self.bdyUnselectedCB)
            ]
        
        self.sensitize()
        
    def currentSkeletonContext(self):
        try:
            return skeletoncontext.skeletonContexts[self.skelwidget.get_value()]
        except KeyError:
            return None

    def widgetChanged(self, interactive): # sb callback from skelwidget
        self.updateBdyList()
        self.sensitize()
        
    def updateBdyList(self):
        skelctxt = self.currentSkeletonContext()
        if skelctxt:
            self.boundarylist.update(skelctxt.allBoundaryNames())
            selbdyname = skelctxt.getSelectedBoundaryName()
            if selbdyname:
                self.boundarylist.set_selection(selbdyname)
        else:
            self.boundarylist.update([])
            
        
    def sensitize(self):
        buttons_alive = self.boundarylist.has_selection()
        self.editbutton.set_sensitive(buttons_alive)
        self.renamebutton.set_sensitive(buttons_alive)
        self.deletebutton.set_sensitive(buttons_alive)
        self.newbutton.set_sensitive(self.currentSkeletonContext() is not None)

    def updateInfo(self):
        self.infotext.backward_delete(self.infotext.get_length())
        bdyname = self.boundarylist.get_value()
        if bdyname is not None:
            skelctxt = self.currentSkeletonContext()
            bdytext = skelctxt.boundaryInfo(bdyname)
            self.infotext.insert_defaults("Boundary %s:\n" % bdyname)
            self.infotext.insert_defaults(bdytext)
        else:
            self.infotext.insert_defaults("No boundary selected.")

    def newBdyConfigCB(self, skelctxt): # sb "new boundary configuration"
        if skelctxt is self.currentSkeletonContext():
            self.sensitize()
            self.updateBdyList()
            self.updateInfo()

    def newBdyCB(self, skelctxt):       # sb bdy creation, removal, or renaming
        if skelctxt is self.currentSkeletonContext():
            self.updateBdyList()
            self.updateInfo()
            self.sensitize()

    def bdySelectedCB(self, skelctxt, name): # sb "boundary selected"
        if skelctxt is self.currentSkeletonContext():
            self.inhibitSignals()
            self.boundarylist.set_selection(name)
            self.uninhibitSignals()
            self.updateInfo()
            self.sensitize()
    def bdyUnselectedCB(self, skelctxt): # sb "boundary unselected"
        if skelctxt is self.currentSkeletonContext():
            self.inhibitSignals()
            self.boundarylist.set_selection(None)
            self.uninhibitSignals()
            self.updateInfo()
            self.sensitize()
    def inhibitSignals(self):
        for sig in self.selectsignals:
            sig.block()
    def uninhibitSignals(self):
        for sig in self.selectsignals:
            sig.unblock()

    def boundarylistCB(self, bdyobj):   # ChooserListWidget callback
        skelctxt = self.currentSkeletonContext()
        if skelctxt:
            bdyname = self.boundarylist.get_value()
            if bdyname:
                skelctxt.selectBoundary(bdyname)
            else:
                skelctxt.unselectBoundary()
        self.sensitize()
        self.updateInfo()

    # Context-sensitivity is built in to the DirectorWidget.
    def newBoundaryCB(self, gtkobj):    # button callback
        menuitem = boundarymenu.boundarymenu.Construct
        nameparam =  menuitem.get_arg('name')
        builderparam = menuitem.get_arg('constructor')
        if parameterwidgets.getParameters(
            nameparam, builderparam, title="New Boundary",scope=self):
            menuitem.callWithDefaults(skeleton=self.skelwidget.get_value())
    
    def modifyBoundaryCB(self, gtkobj): # button callback
        # The parameter widget needs to know what types of modifiers to
        # present -- this can be passed through via the "parameter",
        # which should have it's ".modifiertype" attribute set to the
        # same as the current boundary object.

        menuitem = boundarymenu.boundarymenu.Modify
        modparam = menuitem.get_arg('modifier')

        skelctxt = skeletoncontext.skeletonContexts[
            self.skelwidget.get_value()]
        bdyobj = skelctxt.getBoundary(self.boundarylist.get_value())

        # Hang the extra data we need on the parameter dialog box --
        # it's a widgetscope, so the enclosed widgets can navigate to
        # it easily, and it has the same lifetime as the data in
        # question.  The price of this is that we have to do the
        # dialog operations ourselves.

        dialog_extra = {'modifiertype' : bdyobj.modifiertype,
                       'boundaryname' : self.boundarylist.get_value() }
        
        
        if parameterwidgets.getParameters(
            modparam, title="Boundary modifier", scope=self,
            dialog_data=dialog_extra):
            menuitem.callWithDefaults(skeleton=self.skelwidget.get_value(),
                                      boundary=self.boundarylist.get_value())

    def renameBoundaryCB(self, gtkobj): # button callback
        menuitem = boundarymenu.boundarymenu.Rename
        newname = menuitem.get_arg('name')
        oldname = self.boundarylist.get_value()
        newname.set(oldname)
        if parameterwidgets.getParameters(
            newname, title="New name for this boundary"):
            menuitem.callWithDefaults(
                skeleton=self.skelwidget.get_value(),
                boundary=self.boundarylist.get_value())

    def deleteBoundaryCB(self, gtkobj): # button callback
        menuitem = boundarymenu.boundarymenu.Delete
        menuitem(skeleton=self.skelwidget.get_value(),
                 boundary=self.boundarylist.get_value())

        
    
sbp = SkeletonBoundaryPage()
