# -*- python -*-
# $RCSfile: solverPage.py,v $
# $Revision: 1.55 $
# $Author: langer $
# $Date: 2005/06/20 15:52:46 $


# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.SWIG.common import devel
from oof2.SWIG.common import switchboard
from oof2.common import debug
from oof2.common import labeltree
from oof2.common import microstructure
from oof2.common import utils
from oof2.common import guitop
from oof2.common.IO import mainmenu
from oof2.common.IO import parameter, reporter
from oof2.common.IO.GUI import chooser
from oof2.common.IO.GUI import oofGUI
from oof2.common.IO.GUI import regclassfactory
from oof2.common.IO.GUI import whowidget


## Import of different wrapped solvers
from oof2.SWIG.engine import solver
try:
    from oof2.SWIG.engine.PETSc import petsc_solver
except ImportError:
    pass

from oof2.SWIG.engine import solverdriver
## Import of different wrapped preconditioners
from oof2.SWIG.engine import preconditioner
try:
    from oof2.SWIG.engine.PETSc import petsc_preconditioner
except ImportError:
        pass

from oof2.engine.IO import solvermenu
from oof2.engine import skeletoncontext
from oof2.engine import timedrivers
from gtk import *
import oof2.engine.mesh

OOF = mainmenu.OOF

class SolverPage(oofGUI.MainPage):
    def __init__(self):
        self.sbcallbacks = [
            switchboard.requestCallback("mesh changed", self.meshChanged),
            switchboard.requestCallback("mesh solved",
                                        self.meshSolved),
            switchboard.requestCallback("made reservation",
                                        self.reservationChanged),
            switchboard.requestCallback("cancelled reservation",
                                        self.reservationChanged),
            switchboard.requestCallback(("new who", "Skeleton"),
                                        self.newSkeleton),
            switchboard.requestCallback(("new who", "Mesh"), self.newMesh),
            switchboard.requestCallback(("remove who", "Mesh"), self.removeMesh)
            ]
            
        oofGUI.MainPage.__init__(self, name="Solver",
                                 ordering=240,
                                 tip="Set mesh and time driver to find the solution of active fields and partial differential equations")
        mainbox = GtkVBox()
        self.gtk.add(mainbox)

        hbox = GtkHBox()
        mainbox.add(hbox, expand=0, fill=0)
        centerbox = GtkHBox(spacing=3)
        hbox.add(centerbox, expand=1, fill=0)
        self.meshwidget = whowidget.WhoWidget(oof2.engine.mesh.meshes,
                                              callback=self.meshCB, scope=self)
        label = GtkLabel("Microstructure=")
        label.set_alignment(1.0, 0.5)
        centerbox.add(label, expand=0, fill=0)
        centerbox.add(self.meshwidget.gtk[0], expand=0, fill=0)
        label = GtkLabel("Skeleton=")
        label.set_alignment(1.0, 0.5)
        centerbox.add(label, expand=0, fill=0)
        centerbox.add(self.meshwidget.gtk[1], expand=0, fill=0)
        label = GtkLabel("Mesh=")
        label.set_alignment(1.0, 0.5)
        centerbox.add(label, expand=0, fill=0)
        centerbox.add(self.meshwidget.gtk[2], expand=0, fill=0)
        
        middlebox = GtkVBox()
        mainbox.add(middlebox)

        panes = GtkHPaned()
        middlebox.add(panes)

        statusframe = GtkFrame("Mesh Status")
        panes.pack1(statusframe)
        scroll = GtkScrolledWindow()
        scroll.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        statusframe.add(scroll)
        self.statustext = GtkText()
        self.statustext.set_word_wrap(TRUE)
        scroll.add_with_viewport(self.statustext)

        ## solver component selection
        solverVwindow = GtkVBox()
        panes.pack2(solverVwindow, shrink=0)
        ## solver driver selection
        if devel.devel():
            solverdriverframe = GtkFrame("MatrixDriver")
        else:
            solverdriverframe = GtkFrame("Solver")
        ## panes.pack2(solverdriverframe, shrink=0)
        solverVwindow.add(solverdriverframe)
        self.solverSelectorFactory = regclassfactory.RegisteredClassFactory(
            solverdriver.Driver.registry)
        solverdriverframe.add(self.solverSelectorFactory.gtk)

        if devel.devel():
            ## time_driver selection
            self.time_driverSelectorFactory = regclassfactory.RegisteredClassFactory(
                timedrivers.TimeDriver.registry)
            time_driverframe = GtkFrame("TimeDriver")
            solverVwindow.add(time_driverframe)
            time_driverframe.add(self.time_driverSelectorFactory.gtk)
        
        # Buttons 
        solverbuttons = GtkHBox()
        if devel.devel():
            self.resetButton = GtkButton("Reset")
            self.resetButton.connect("clicked", self.resetCB)
            guitop.top().tooltip(self.resetButton, "set fields back to their (unsolved) initial state")
            solverbuttons.add(self.resetButton)
        self.solveButton = GtkButton("Solve")
        self.solveButton.connect("clicked", self.solveCB)
        guitop.top().tooltip(self.solveButton, "Apply solver to finite element mesh and associated equations and fields")
        solverbuttons.add(self.solveButton)
        middlebox.add(solverbuttons, expand=0, fill=0)
        
        self.sensitize()



        
    def show(self):
        debug.mainthreadTest()
        self.gtk.show_all()
        self.update()

    #######################

    def currentMSName(self):
        path = labeltree.makePath(self.currentFullMeshName())
        return path[0]
    def currentMS(self):
        msname = self.currentMSName()
        if msname:
            return microstructure.microStructures[msname].getObject()

    def currentSkeletonName(self):
        path = labeltree.makePath(self.currentFullMeshName())
        return path[1]
    def currentSkeleton(self):
        ms = self.currentMS()
        skelname = self.currentSkeletonName()
        if skelname:
            return skeletoncontext.getSkeleton(ms, skelname)

    def currentFullMeshName(self):
        return self.meshwidget.get_value()
    
    def currentMeshName(self):
        path = labeltree.makePath(self.currentFullMeshName())
        return path[2]
    def currentMeshContext(self):
        try:
            return oof2.engine.mesh.meshes[self.currentFullMeshName()]
        except KeyError:
            return None
    def currentMesh(self):
        meshcontext = self.currentMeshContext()
        if meshcontext is not None:
            return meshcontext.getObject()

    def sensitize(self):
        debug.mainthreadTest()
        msok = self.currentMS() is not None
        skelok = msok and self.currentSkeleton() is not None
        mesh = self.currentMesh()
        meshcontext = self.currentMeshContext()
        meshok = skelok and (mesh is not None)
        self.solveButton.set_sensitive(
            meshok and                  # we have a mesh
            not meshcontext.query_reservation() and # ... and it's not busy
            meshcontext.materialsConsistent() # ... and it's computable
            )
        if devel.devel():
            self.resetButton.set_sensitive(
                meshok and                  # we have a mesh
                not meshcontext.query_reservation() # ... and it's not busy
                )
            
    def update(self):
        self.set_state(self.currentFullMeshName())
        self.updateStatus()

    def updateStatus(self):
        meshcontext = self.currentMeshContext()
        self.statustext.backward_delete(self.statustext.get_length())
        if meshcontext is not None:
            self.statustext.insert_defaults(meshcontext.status())
        else:
            self.statustext.insert_defaults("No mesh!")
        
    def set_state(self, meshpath):
        path = labeltree.makePath(meshpath)
        self.meshwidget.set_value(path)
        mesh = self.currentMesh()
        

    def newSkeleton(self, skeletonname): # switchboard ("new who", "Skeleton")
        self.sensitize()

    def newMesh(self, meshname):        # switchboard ("new who", "Mesh")
        self.set_state(meshname)
        self.sensitize()

    def meshCB(self, meshpath):         # WhoWidget callback
        self.set_state(meshpath)
        self.sensitize()

    def removeMesh(self, path):         # switchboard ("remove who", "Mesh")
        self.update()
        self.sensitize()

    def reservationChanged(self, whoobj):
        # switchboard "made reservation", "cancelled reservation"
        if  self.currentFullMeshName() == whoobj.path():
            self.sensitize()

    def meshChanged(self, meshcontext):
        if self.currentMeshContext() is meshcontext:
            self.sensitize()
            self.updateStatus()

    def meshSolved(self, meshcontext):
        if self.currentMeshContext() is meshcontext:
            self.sensitize()
            self.updateStatus()

    ###############################

    def solveCB(self, gtkobj):    # "Solve" button callback.
        the_mesh = self.currentFullMeshName()        
        solver_selection = self.solverSelectorFactory.get_value()
        if devel.devel():
            time_driver_selection = self.time_driverSelectorFactory.get_value()
            OOF.Solver.Solve(mesh=the_mesh, matrixdriver=solver_selection, timedriver=time_driver_selection)
        else:
            OOF.Solver.Solve(mesh=the_mesh, solver=solver_selection)
            
    def resetCB(self, gtkobj):
        the_mesh = self.currentFullMeshName()
        OOF.Solver.Reset(mesh=the_mesh)
        
#####################################################################

solverpage = SolverPage()



