# -*- python -*-
# $RCSfile: fiddlenodesbaseParallel.py,v $
# $Revision: 1.37 $
# $Author: haan $
# $Date: 2005/05/24 19:44:00 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.common import debug
from oof2.common import primitives
from oof2.common import registeredclass
from oof2.common.IO import oofmenu
from oof2.common.IO import reporter
from oof2.common.IO import parameter
from oof2.engine import deputy
from oof2.engine import skeletonmodifier
from oof2.SWIG.common.IO import progressbar
from oof2.SWIG.common import mpitools
from oof2.SWIG.common import switchboard
from oof2.engine import  skeletoncontext
from oof2.engine.IO import skeletonmenu
from oof2.engine.IO import skeletonIPC
import math
import random

_rank = mpitools.rank()
_size = mpitools.size()
## In parallel _coreProcess tells the back end
## which Who (context) will be modified.

def _coreProcessCall(self, context, activenodes):
    global _rank
    if _rank == 0:
        ## front end tells back-end to mirror operation
        ## back-end moves only those nodes it owns
        skelname = context.getObject().name()
        skeletonIPC.smenu.FiddleNodes(skeleton = skelname, pbar = self.pbar_type, criterion = self.criterion, fiddlenodesmove = self.movedPosition,  target = self.targets, temperature = self.T) ## back-end part
        try:
            context.begin_writing()
            internal_nodes = _get_internal_nodes(activenodes)
            _coreProcess(self, context, internal_nodes)
            _broadcast_loop(internal_nodes, context)
            _set_energy_parameters(context, self.nok, self.nbad, self.deltaE) ## receive energy change and nodes moved
            sharednodes = _get_shared_nodes(activenodes) ## create list of shared-nodes
            mpitools.mpi_barrier()
            _coreProcess(self, context, sharednodes) ## move nodes
            _broadcast_send_node_positions(sharednodes, 0) ## tell back-end about shared nodes
            skeleton = context.getObject()
            skeleton.timestamp.increment()
            switchboard.notify("redraw")
        finally:
            context.end_writing()
        mpitools.mpi_barrier()
        

def _coreProcess(self, context, activenodes):
    ## debug.fmsg(activenodes)
    ## The core process executes common actions on each
    ## processor.
    ## postprocessing distributes and coordinates the job
    global _rank
    ## everybody's work
    skeleton = context.getObject()
    pBar = progressbar.getProgress()
    self.totalEnergy = skeleton.energyTotal(self.criterion.alpha)
    self.nok = self.nbad = 0
    self.deltaE = 0.0
    _randomize(activenodes)
    j = 0
    not_moved = []
    for node in activenodes:
        change = deputy.DeputyProvisionalChanges()
        change.moveNode(node, self.movedPosition(skeleton, node), skeleton)
        bestchange = self.criterion([change], skeleton)
        if bestchange is not None:
            self.nok += 1
            self.deltaE += bestchange.deltaE(skeleton, self.criterion.alpha)
            bestchange.accept(skeleton)
                
        # Failed to meet the specified criterion ... but
        elif bestchange is None and not change.illegal(skeleton) and \
                 not self.criterion.hopeless() and self.T>0.0:
            diffE = change.deltaE(skeleton, self.criterion.alpha)
            if math.exp(-diffE/self.T) > random.random():
                self.nok += 1
                self.deltaE += diffE
                change.accept(skeleton)
            else:
                self.nbad += 1
                not_moved.append(node)
        else:
            self.nbad += 1
            not_moved.append(node)
        ## progress bars stuff
        if _rank == 0:
            if pBar.query_stop():
                pBar.set_failure()
                pBar.set_message("Failed")
                break
            else:
                if self.pbar_type != "continuous":
                ## this is for "shuttle" mode?
                    n = 100
                    j+=17 ## magic number for going back-and-forth
                    if j >= 100:
                        j = 0
                    pBar.set_progress(1.0*j/n)
    ## not part of for-loop
    for nd in not_moved:
        activenodes.remove(nd)
        

class BackEndFiddleNodes:
    def __init__(self, pbar_type, criterion, fiddlenodesmove, target, temperature):
        self.pbar_type = pbar_type
        self.alpha = criterion.alpha
        self.criterion = criterion 
        self.T = temperature
        self.movedPosition = fiddlenodesmove
        self.target = target
        self.nok = 0
        self.nbad = 0
        self.deltaE = 0
        self.totalEnergy = 0
    def coreProcess(self, context, activenodes):
        pass ## overwritten in the next line
BackEndFiddleNodes.coreProcess = _coreProcess



def _randomize(nodelist):
    random.shuffle(nodelist)

def shared(node):
    global _rank
    for el in node._elements:
        if el.ID.get_rank() != node.ID.get_rank():
            ##debug.fmsg("SHARED index=", node.ID.get_index(), " rank=", node.ID.get_rank(), " machine ID=", _rank)
            return 1
    ##debug.fmsg("NOT SHARED index=", node.ID.get_index(), " rank=", node.ID.get_rank(), " machine ID=", _rank)    
    return 0

def _get_shared_nodes(nodelist):
    bnodes = []
    for node in nodelist:
        if shared(node):
            if node not in bnodes:
                ## debug.fmsg("SHARED index=", node.ID.get_index(), " rank=", node.ID.get_rank(), " machine ID=", _rank)
                bnodes.append(node)
    return bnodes ## returns list of nodes that are on a boundary

def _get_internal_nodes(nodelist):
    global _rank
    inodes = []
    for node in nodelist:
        if not shared(node):
            if node.ID.get_rank() == _rank:
                if node not in inodes:
                    ## debug.fmsg("NOT SHARED index=", node.ID.get_index(), " rank=", node.ID.get_rank(), " machine ID=", _rank)
                    inodes.append(node)
    ## returns list of nodes that are not shared
    ## and belong to the processor that owns the
    ## writing permissions
    return inodes 


def _extract_index_and_positions(nodelist):
    relevantIndexList = [] ## list of node indices
    relevantPositionList = [] ## list of node positions
    global _rank
    for nd in nodelist:
            relevantIndexList.append(nd.ID.get_index())
            relevantIndexList.append(nd.ID.get_rank())
            relevantPositionList.append(nd.position()[0])
            relevantPositionList.append(nd.position()[1])
    return (relevantIndexList, relevantPositionList)

def _set_index_and_positions(context, indexlist, positionlist, the_rank):
    ## debug.fmsg("indexlist=", indexlist)
    nodelist = context.getObject().nodes
    skel = context.getObject()
    nodedict = context.getObject().node_dictionary
    N = len(indexlist)/2
    global _rank
    for i in range(N):
        try:
            node = nodedict[(indexlist[2*i], indexlist[2*i + 1])]
            ## node.moveTo(primitives.Point(positionlist[2*i], positionlist[2*i + 1]))
            skel.moveNodeTo(node, primitives.Point(positionlist[2*i], positionlist[2*i + 1]))
            ## debug.fmsg("found key ", (indexlist[2*i], indexlist[2*i + 1]), _rank)
            ##debug.fmsg("found a non-trivial node!= ", (indexlist[i], the_rank), node)
        except KeyError:
            ##debug.fmsg("key not found. ", (indexlist[2*i], indexlist[2*i + 1]), _rank)
            pass
            
        
        
def _send_nodes(nodelist, destination):
    ## in each processor, nodelist may have nodes that the machine does not own
    ## Therefore, only the relevant nodes should be sent
    relevantIndexList, relevantPositionList = _extract_index_and_positions(nodelist)
    ## send is done in two parts:
    ## first indices and then positions
    ## debug.fmsg(relevantIndexList, relevantPositionList)
    mpitools.send_vector_of_ints(relevantIndexList, destination)
    mpitools.send_vector_of_doubles(relevantPositionList, destination)

def _receive_node_positions(context, origin):
    index_list = mpitools.receive_vector_of_ints(origin)
    position_list = mpitools.receive_vector_of_doubles(origin)
    ## debug.fmsg(index_list, position_list)
    _set_index_and_positions(context, index_list, position_list, origin)


def _broadcast_send_node_positions(nodelist, destination):
    relevantIndexList, relevantPositionList = _extract_index_and_positions(nodelist)
    ## send is done in two parts:
    ## first indices and then positions
    mpitools.broadcast_vector_of_ints(relevantIndexList, destination)
    mpitools.broadcast_vector_of_doubles(relevantPositionList, destination)
    
def _broadcast_receive_node_positions(context, origin):
    indexlist = mpitools.receive_broadcast_vector_of_ints(origin)
    positionlist = mpitools.receive_broadcast_vector_of_doubles(origin)
    ## debug.fmsg(indexlist, positionlist)
    _set_index_and_positions(context, indexlist, positionlist, origin)

def _broadcast_loop(nodelist, context):
    global _rank
    global _size
    for source in range(_size):
        if source == _rank: ## if this processor's turn to send
            _broadcast_send_node_positions(nodelist, source)
        else:
            _broadcast_receive_node_positions(context, source)
        ## mpitools.mpi_barrier()
def _set_energy_parameters(context, nok, nbad, deltaE):
    global _rank
    global _size
    for source in range(_size):
        if _rank == 0 and source != 0:
            parms = mpitools.receive_vector_of_doubles(source)
            nok += parms[0]
            nbad += parms[1]
            deltaE += parms[2]
    if _rank != 0:
        parms = [nok, nbad, deltaE]
        mpitools.send_vector_of_doubles(parms, 0)
        
def _backEndCoreProcess(menuitem, skeleton, pbar, criterion, fiddlenodesmove, target, temperature):
    ## back-end sets self.pbar_type as continuous
    global _rank
    if _rank == 0:
        return
    context = skeletoncontext.skeletonContexts[skeleton]
    context.reserve()
    try:
        context.begin_writing()
        try:
            ## randomization of nodes is made inside coreProcess
            philnodes = BackEndFiddleNodes(pbar, criterion, fiddlenodesmove, target, temperature)
            internal_nodes = _get_internal_nodes(philnodes.target(context))
            philnodes.coreProcess(context, internal_nodes) ## move interior nodes

            _broadcast_loop(internal_nodes, context)
            _set_energy_parameters(context, philnodes.nok, philnodes.nbad, philnodes.deltaE)
            mpitools.mpi_barrier()
            ## finally, front-end sends to back-end, and
            ## back-end synchronizes boundary node positions
            _broadcast_receive_node_positions(context, 0)
            ## after all these changes, the timestamp and GUI should be updated
            skeletonobj = context.getObject()
            skeletonobj.timestamp.increment()
        finally:
            context.end_writing()
    finally:
        context.cancel_reservation()
    mpitools.mpi_barrier()

from oof2.fiddlenodesbase import FiddleNodesMovePosition
from oof2.fiddlenodesbase import FiddleNodesTargets
skeletonIPC.smenu.addItem(oofmenu.OOFMenuItem(
    'FiddleNodes', 
    callback = _backEndCoreProcess,
    secret=1,
    no_log=1,
    threadable = oofmenu.PARALLEL_UNTHREADABLE,
    params=[parameter.StringParameter('skeleton'),
            parameter.StringParameter('pbar'),
            parameter.RegisteredParameter('criterion',
                                          skeletonmodifier.SkelModCriterion),
            parameter.RegisteredParameter('fiddlenodesmove',
                                          FiddleNodesMovePosition),
            parameter.RegisteredParameter('target', FiddleNodesTargets),
            parameter.FloatParameter('temperature')] ))
